
/* Chrysalide - Outil d'analyse de fichiers binaires
 * block.h - prototypes pour l'encadrement des instructions par blocs
 *
 * Copyright (C) 2016-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ANALYSIS_DISASS_BLOCK_H
#define _ANALYSIS_DISASS_BLOCK_H


#include <glib-object.h>


#include "../../arch/instruction.h"
#include "../../common/bits.h"



/* ------------------------ MISE EN PLACE DES BLOCS BASIQUES ------------------------ */


#define G_TYPE_BASIC_BLOCK              g_basic_block_get_type()
#define G_BASIC_BLOCK(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), g_basic_block_get_type(), GBasicBlock))
#define G_IS_BASIC_BLOCK(obj)           (G_TYPE_CHECK_INSTANCE_TYPE((obj), g_basic_block_get_type()))
#define G_BASIC_BLOCK_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_BASIC_BLOCK, GBasicBlockClass))
#define G_IS_BASIC_BLOCK_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_BASIC_BLOCK))
#define G_BASIC_BLOCK_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_BASIC_BLOCK, GBasicBlockClass))


/* Description d'un bloc basique d'instructions (instance) */
typedef struct _GBasicBlock GBasicBlock;

/* Description d'un bloc basique d'instructions (classe) */
typedef struct _GBasicBlockClass GBasicBlockClass;


/* Indique le type défini pour un bloc d'instructions basique. */
GType g_basic_block_get_type(void);

/* Crée un bloc basique d'exécution d'instructions. */
GBasicBlock *g_basic_block_new(GArchInstruction *, GArchInstruction *, const bitfield_t *);

/* Fournit les instructions limites d'un bloc basique. */
void g_basic_block_get_boundary(const GBasicBlock *, GArchInstruction **, GArchInstruction **);

/* Fournit les adresses limites d'un bloc basique. */
void g_basic_block_get_boundary_addresses(const GBasicBlock *, vmpa2t *, vmpa2t *);

/* Fournit le rang du bloc basique dans le flot d'exécution. */
unsigned int g_basic_block_get_rank(const GBasicBlock *);

/* Définit le rang du bloc basique dans le flot d'exécution. */
void g_basic_block_set_rank(GBasicBlock *, unsigned int);

/* Indique la liste des blocs de code dominés. */
const bitfield_t *g_basic_block_get_domination(const GBasicBlock *);



/* ------------------------- REGROUPEMENT EN LISTE DE BLOCS ------------------------- */


#define G_TYPE_BLOCK_LIST               g_block_list_get_type()
#define G_BLOCK_LIST(obj)               (G_TYPE_CHECK_INSTANCE_CAST((obj), g_block_list_get_type(), GBlockList))
#define G_IS_BLOCK_LIST(obj)            (G_TYPE_CHECK_INSTANCE_TYPE((obj), g_block_list_get_type()))
#define G_BLOCK_LIST_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_BLOCK_LIST, GBlockListClass))
#define G_IS_BLOCK_LIST_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_BLOCK_LIST))
#define G_BLOCK_LIST_GET_CLASS(obj)     (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_BLOCK_LIST, GBlockListClass))


/* Description d'une liste de blocs basiques (instance) */
typedef struct _GBlockList GBlockList;

/* Description d'une liste de blocs basiques (classe) */
typedef struct _GBlockListClass GBlockListClass;


/* Indique le type défini pour une liste de blocs basiques. */
GType g_block_list_get_type(void);

/* Crée une liste de blocs basiques. */
GBlockList *g_block_list_new(size_t);

/* Compte le nombre de blocs basiques représentés. */
size_t g_block_list_count_blocks(const GBlockList *);

/* Ajoute un bloc basique à une liste définie. */
void g_block_list_add_block(GBlockList *, GBasicBlock *, size_t);

/* Fournit un bloc basique à d'une liste définie. */
GBasicBlock *g_block_list_get_block(const GBlockList *, size_t );

/* Fournit l'indice d'un bloc basique d'une liste de blocs. */
size_t g_block_list_get_index(const GBlockList *, GBasicBlock *);

/* Recherche un bloc basique selon une première instruction. */
GBasicBlock *g_block_list_find_by_starting_instr(const GBlockList *, GArchInstruction *);



#endif  /* _ANALYSIS_DISASS_BLOCK_H */
